import "./root";
export const bk_canvas = "bk-canvas";
export const bk_canvas_underlays = "bk-canvas-underlays";
export const bk_canvas_overlays = "bk-canvas-overlays";
export const bk_canvas_events = "bk-canvas-events";
//# sourceMappingURL=canvas.js.map