import "./root";
import * as _a from "../core/dom";
_a.styles.append(".bk-root .bk-clearfix:before,\n.bk-root .bk-clearfix:after {\n  content: \"\";\n  display: table;\n}\n.bk-root .bk-clearfix:after {\n  clear: both;\n}\n");
export const bk_clearfix = "bk-clearfix";
//# sourceMappingURL=clearfix.js.map